from django.forms.widgets import Widget
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from djinn_core.utils import urn_to_object


TPL = 'djinn_forms/snippets/sharewidget.html'


class ShareWidget(Widget):

    """ Widget for handling relations to other content.
    The following extra attributes are supported:
     * content_types Allowed content types for this relation
     * relation_type Relation type to use for creating the actual relation
     * src_obj Source object for relation
    """

    def value_from_datadict(self, data, files, name):

        """ The data may contain a list of objects to remove, and
        objects to add. Both are prefixed by the field name. The
        returned value is a dict with 'rm' and 'add' lists, that list
        the """

        result = {'rm': [], 'add': []}

        for item in data.get("%s_rm" % name, "").split(";;"):

            obj = urn_to_object(item)

            if obj:
                result['rm'].append(obj)

        for item in data.get("%s_add" % name, "").split(";;"):

            obj = urn_to_object(item)

            if obj:
                result['add'].append(obj)

        return result

    def render(self, name, value, attrs=None):

        url = self.attrs.get("search_url", reverse("djinn_forms_relatesearch"))
        url = "%s?content_types=%s&searchfield=%s" % (
            url,
            ",".join(self.attrs['content_types']),
            self.attrs.get("searchfield", "title")
            )

        context = {'name': name,
                   'hint': self.attrs.get("hint", ""),
                   # Translators: djinn_forms relate add button label
                   'add_label': self.attrs.get("add_label", _("Add")),
                   'value': value,
                   'search_minlength': self.attrs.get("search_minlength", 2),
                   'search_url': url
                   }

        html = render_to_string(TPL, context)

        return mark_safe(u"".join(html))



from django import forms
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string


class ImageWidget(forms.widgets.Widget):

    """ Image upload widget """

    template_name = "djinn_forms/snippets/imagewidget.html"

    def render(self, name, value, attrs=None):

        try:
            value = self.model.objects.get(pk=value)
        except:
            pass

        context = {
            'name': name,
            'widget': self,
            'show_progress': True,
            'multiple': False,
            'value': value
            }

        context.update(self.attrs)

        if value:
            accessor = "get_%s_url" % self.attrs.get('size', 'thumbnail')
            try:
                context['image_url'] = getattr(value, accessor)()
            except:
                pass

        if attrs:
            context.update(attrs)

        return mark_safe(render_to_string(self.template_name, context))



from django import forms
from djinn_forms.fields.share import ShareField


class ShareMixin(object):

    """ When using share fields, handle these in the save """

    def save_shares(self, commit=True):

        for f_name, field in self.fields.items():
            if isinstance(field, ShareField):

                val = self.cleaned_data.get(f_name, {'rm': [], 'add': []})
                field.save_share(self.instance, val, commit)

    def init_share_fields(self):

        for f_name, field in self.fields.items():
            if isinstance(field, ShareField):
                self.fields[f_name].instance = self.instance



from django.forms.fields import Field
from djinn_forms.widgets.image import ImageWidget


class ImageField(Field):

    """ Convert given value to image """

    widget = ImageWidget

    def __init__(self, model=None, **kwargs):

        super(ImageField, self).__init__(**kwargs)

        self.model = model

    def prepare_value(self, data):

        try:
            return self.model.objects.get(pk=data)
        except:
            return None

    def to_python(self, value):

        try:
            return self.model.objects.get(pk=value)
        except:
            return None



import json
import magic
import os
from tempfile import mkstemp
import uuid
import logging as log
import mimetypes
from shutil import copyfile
from django.db.models import get_model
from django.template.loader import render_to_string
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import View
from django.http import HttpResponse
from django.conf import settings
from django.core.files import File


class UploadView(View):

    """ View that enables Ajax style upload of attachments """

    def is_image(self, mime_type):

        """ Check whether the mime_type is 'image'-ish """
        try:
            return mime_type.split("/")[0] == "image"
        except:
            return False

    @csrf_exempt
    def post(self, request, *args, **kwargs):

        """ File upload view. If attachment_id is provided in the
        request params, assume it is an edit of an existing
        attachment. """

        temp_file = None
        file_name = None

        attachments = []
        attachment_type = request.REQUEST.get(
            "attachment_type",
            "djinn_contenttypes.DocumentAttachment")

        if attachment_type in ["image", "avatar"]:
            model = get_model("pgcontent", "ImageAttachment")
        elif attachment_type == "document":
            model = get_model("pgcontent", "DocumentAttachment")
        else:
            parts = attachment_type.split('.')
            model = get_model(parts[0], parts[-1])

        try:
            attachment_id = int(request.REQUEST.get("attachment_id", None))
        except:
            attachment_id = None

        for filename in request.FILES.keys():

            fd, temp_file = self.create_temp_file(request.FILES[filename])
            file_name = request.FILES[filename].name
            mime_type = request.FILES[filename].content_type or \
                magic.Magic(mime=True).from_file(temp_file) or \
                mimetypes.guess_type(file_name)[0]

            # connect the uploaded file to an Attachment instance
            #
            log.debug("Uploaded file mime type: %s" % mime_type)

            relative_file_name = self.generate_relative_file_path(file_name,
                                                              mime_type,
                                                              attachment_type)

            if attachment_type in ["image", "avatar"]:

                if not self.is_image(mime_type):

                    log.error("Upload is not an image.")

                    response = HttpResponse(
                        json.dumps({'success': False,
                                    'msg': "Upload is not an image"
                                    }))

                    return response

                if attachment_id:
                    attachment = model.objects.get(pk=attachment_id)
                    attachment.original_filename = file_name
                else:
                    attachment = model(
                        mime_type=mime_type,
                        title=file_name,
                        original_filename=file_name
                        )

                if attachment_type == "avatar":
                    path = "avatars/%s" % relative_file_name
                else:
                    path = "images/%s" % relative_file_name

                attachment.image.save(path, File(open(temp_file)))
                attachment.save()
            elif attachment_type == "document":
                store_path = os.path.join("documents", relative_file_name)

                self.store(temp_file, store_path)

                if attachment_id:
                    attachment = model.objects.get(
                        pk=attachment_id)
                    attachment.file = store_path
                    attachment.original_filename = file_name
                else:
                    attachment = model.objects.create(
                        file=store_path,
                        mime_type=mime_type,
                        title=file_name,
                        original_filename=file_name
                        )

                attachment.save()
            else:

                attachment = model()

                path = "images/%s" % relative_file_name
                attachment.image.save(path, File(open(temp_file)))
                attachment.save()

            attachments.append(attachment)

        # TODO: always return JSON and let the widget handle the rendering
        #
        if attachment_type not in ["document", "image", "avatar"]:
            attachment_type = "image"

        # Send back results to the client. Client should 'handle'
        # file_id etc.
        #
        context = {}

        context["attachment_ids"] = [att.id for att in attachments]

        if self.request.REQUEST.get("edit_type", "") == "field":
            if attachment_type == "image":
                template = "pgcontent/snippets/image.html"
            elif attachment_type == "avatar":
                template = "pgcontent/snippets/avatar.html"
            else:
                template = "pgcontent/snippets/document.html"
        else:
            if attachment_type == "image":
                template = "pgcontent/snippets/imageattachments.html"
            else:
                template = "pgcontent/snippets/documentattachments.html"

        context['html'] = render_to_string(template,
                                           {'attachments': attachments})

        context.update(self.extra_json_context(attachments))

        response = HttpResponse(json.dumps(context), content_type="text/plain")

        return response

    def extra_json_context(self, attachments):

        return {}

    def generate_relative_file_path(self, file_name, mime_type,
                                    attachment_type):

        """ Check on file type, and generate path accordingly """

        fn, file_extension = os.path.splitext(file_name)

        # Create a random filename, with the proper extension.
        #
        file_id = str(uuid.uuid4())
        file_name = os.path.join(
            file_id[:2],
            "%s%s" % (file_id, file_extension))

        return file_name

    def create_temp_file(self, data):

        fd, path = mkstemp()

        block = data.read(1024)
        while block:
            os.write(fd, block)
            block = data.read(1024)

        os.close(fd)
        return fd, path

    def store(self, temp_file, file_name):

        file_name = os.path.join(settings.MEDIA_ROOT, file_name)

        if not os.path.exists(os.path.dirname(file_name)):
            os.makedirs(os.path.dirname(file_name))

        copyfile(temp_file, file_name)
        os.unlink(temp_file)



import json
from django.http import HttpResponse
from django.views.generic import View
from haystack.query import SearchQuerySet
from haystack.inputs import Raw
from djinn_core.utils import object_to_urn


class RelateSearch(View):

    """ Find related stuff, using autocomplete """

    def get(self, request, term=None):

        term = term or request.GET.get('term', '')

        if not term:
            return HttpResponse(
                json.dumps([]),
                mimetype='application/json')

        search_field = "%s__contains" % request.GET.get("searchfield",
                                                        "title")

        ct_parameter = request.GET.get("ct_searchfield", "meta_ct")

        ct_search_field = "%s__in" % ct_parameter

        content_types = request.GET.get(ct_parameter, '').split(",")

        _filter = {search_field: Raw("*%s*" % term)}

        if content_types:
            _filter[ct_search_field] = content_types

        sqs = SearchQuerySet().filter(**_filter)

        results = []

        for res in [res for res in sqs if res.object]:
            results.append({"label": unicode(res.object),
                            "value": object_to_urn(res.object)
                            })

        return HttpResponse(json.dumps(results), mimetype='application/json')



from django.forms.widgets import Widget
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string
from djinn_core.utils import get_object_by_ctype_id, object_to_urn, \
    urn_to_object


class LinkWidget(Widget):

    """ Link widget for internal and external links """

    def value_from_datadict(self, data, files, name):

        value = data.get(name, "").split("::")

        if not len(value) == 4:
            return None

        if value[1] and value[2]:
            obj = get_object_by_ctype_id(value[1], value[2])
            return object_to_urn(obj) + "::" + value[3]
        else:
            return value[0] + "::" + value[3]

    def render(self, name, value, attrs=None):

        lexval = ""

        if value:
            url = value.split("::")[0]

            if url.startswith("urn"):
                lexval = urn_to_object(url).title
            else:
                lexval = url or ""

        context = {'name': name,
                   'lexical_value': lexval,
                   'value': value or "",
                   }

        html = render_to_string('djinn_forms/snippets/link_widget.html',
                                context)

        return mark_safe(u"".join(html))



import re
from django import forms
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string
from django.core.urlresolvers import reverse


class AttachmentWidget(forms.widgets.Widget):

    """ Extended file widget, using the jquery file uploader. """

    def __init__(self, model, template, attrs=None):

        super(AttachmentWidget, self).__init__(attrs=attrs)

        self.model = model
        self.template = template

    def value_from_datadict(self, data, files, name):

        """ Conservatively try to get the submitted data """

        _val = None

        if self.attrs.get("multiple", False):
            _val = [int(val) for val in data.get(name, "").split(",") if val]
        else:
            try:
                _val = int(re.search(r'([0-9]+)', data.get(name)).group(1))
            except:
                pass

        return _val

    def _normalize_value(self, value):

        if self.attrs.get("multiple", False):
            if not hasattr(value, "__iter__"):
                value = [value]

            return filter(lambda x: self.model.objects.filter(pk=x).exists(),
                          value)
        else:
            return value

    def render(self, name, value, attrs=None):

        value = self._normalize_value(value)

        context = {
            'name': name,
            'button_label': self.attrs.get("button_label", ""),
            'widget': self,
            'show_progress': True,
            'attachment_type': "%s.%s" % (self.model._meta.app_label,
                                          self.model.__name__),
            'multiple': False,
            'upload_url': reverse("djinn_forms_fileupload")
            }

        if self.attrs.get("multiple", False):
            context['value'] = ",".join([str(val) for val in value])
        else:
            context['value'] = value or ""

        attachments = []

        if value:
            if hasattr(value, "__iter__"):
                for val in value:
                    try:
                        attachments.append(self.model.objects.get(pk=val))
                    except:
                        pass  # too bad
            else:
                try:
                    attachments.append(self.model.objects.get(pk=value))
                except:
                    pass  # too bad

        context['attachments'] = attachments
        context.update(self.attrs)

        if attrs:
            context.update(attrs)

        return mark_safe(render_to_string(self.template, context))



from pgauth.models import Role
from djinn_core.utils import object_to_urn
from relate import RelateField, UpdateRelations, UpdateRelation


def _profile_to_user_or_group(profile):

    return getattr(profile, "user", getattr(profile, "usergroup", None))


class UpdateRoles(UpdateRelations):

    def __init__(self, instance, data, role):
        self.instance = instance
        self.rms = data.get('rm', [])
        self.adds = data.get('add', [])
        self.role = role

    def update(self):

        role = Role.objects.get(name=self.role)

        # Unrelate
        for profile in self.rms:
            self.instance.rm_local_role(role,
                                        _profile_to_user_or_group(profile))

        # Relate
        for profile in self.adds:
            self.instance.add_local_role(role,
                                         _profile_to_user_or_group(profile))


class UpdateRole(UpdateRelation):

    def __init__(self, instance, data, role):
        self.instance = instance
        self.tgt = data
        self.role = role

    def update(self):

        role = Role.objects.get(name=self.role)

        if self.tgt:
            self.instance.rm_local_role(role)
            self.instance.add_local_role(role,
                                         _profile_to_user_or_group(self.tgt))


class LocalRoleField(RelateField):

    """ Field for assigning roles to users or groups on a given object """

    updater = UpdateRoles

    def __init__(self, role_id, content_types, *args, **kwargs):

        self.role = role_id
        super(LocalRoleField, self).__init__(self.role, content_types, *args,
                                        **kwargs)

    def prepare_value(self, data):

        """ Get the existing local roles for the given role.  If
        data['rm'] has values, filter those out of the result.  If
        data['add'] has values, add those to the result
        """

        # We have an initial value set...
        #
        if data and not "add" in data.keys():
            return data
            
        role = Role.objects.get(name=self.role)

        roles = self.instance.get_local_roles(role_filter=[role.name])

        users_or_groups = [(lrole.user or lrole.usergroup)
                           for lrole in roles
                           if lrole.user or lrole.usergroup]

        users_or_groups = [u.profile for u in users_or_groups]

        try:
            users_or_groups = filter(lambda x: x not in data['rm'],
                                     users_or_groups)
        except:
            pass

        try:
            users_or_groups += data['add']
        except:
            pass

        return [{'label': unicode(profile),
                 'value': object_to_urn(profile)} for profile in
                users_or_groups]


class LocalRoleSingleField(LocalRoleField):

    updater = UpdateRole

    def prepare_value(self, data):

        """ Get existing role, if it's there """

        # We have an initial value set...
        #
        if data:
            return {'label': str(data), 'value': object_to_urn(data)}

        value = super(LocalRoleSingleField, self).prepare_value(data)

        if value:
            return value[0]
        else:
            return None



from django.core.urlresolvers import reverse
from djinn_core.utils import urn_to_object
from base import InOutWidget


class RelateWidget(InOutWidget):

    """
    Widget for handling relations to other content. The following
    extra attributes are supported:

     * content_types Allowed content types for this relation as list
     * searchfield Look for this field in the searchengine
     * ct_searchfield Use this field to check on the contenttype. Defaults
       to meta_ct.
     * search_minlength Start autosearch when length is this or more
     * search_url Use this URL as search base. Defaults to reverse of
       'djinn_forms_relatesearch'
    *  template_name path to teh rendering template. Defaults to
       djinn_forms/snippets/relatewidget.html

    To actually save the data that this widget produces, you need to
    make your form extend the djinn_forms.forms.RelateMixin and call
    the methods in that form as documented there.

    TODO: add unique setting to make sure the related object is not already in
    """

    initial = False

    @property
    def template_name(self):
        return self.attrs.get('template_name',
                              'djinn_forms/snippets/relatewidget.html')

    def convert_item(self, item):

        return urn_to_object(item)

    def build_attrs(self, extra_attrs=None, **kwargs):

        final_attrs = super(RelateWidget, self).build_attrs(
            extra_attrs=extra_attrs, **kwargs)

        ct_searchfield = self.attrs.get("ct_searchfield", "meta_ct")

        url = self.attrs.get("search_url", reverse("djinn_forms_relatesearch"))
        url = "%s?%s=%s&searchfield=%s&ct_searchfield=%s" % (
            url,
            ct_searchfield,
            ",".join(self.attrs['content_types']),
            self.attrs.get("searchfield", "title_auto"),
            self.attrs.get("ct_searchfield", "meta_ct"),
            )

        final_attrs.update(
            {'search_minlength': self.attrs.get("search_minlength", 2),
             'search_url': url
             })

        final_attrs['initial'] = self.initial

        return final_attrs


class RelateSingleWidget(RelateWidget):

    """ Relate widget where only one relation is allowed """

    template_name = 'djinn_forms/snippets/relatesinglewidget.html'

    def value_from_datadict(self, data, files, name):

        """ The data may contain a list of objects to remove, and
        objects to add. Both are prefixed by the field name. The
        returned value is a dict with 'rm' and 'add' lists, that list
        the """
        
        result = super(RelateSingleWidget, self).value_from_datadict(
            data, files, name)

        result = result.get('add', [])

        if len(result):
            return result[0]
        else:
            return None



from django.conf.urls.defaults import url, patterns
from djinn_forms.views.fileupload import UploadView
from djinn_forms.views.relate import RelateSearch
from django.views.decorators.csrf import csrf_exempt


urlpatterns = patterns(
    "",
    
    url(r'^fileupload$',
        csrf_exempt(UploadView.as_view()),
        name='djinn_forms_fileupload'
        ),

    url(r'^searchrelate$',
        RelateSearch.as_view(),
        name='djinn_forms_relatesearch'
        ),

    
    )



from base import BaseWidget
from datetime import datetime


class DateTimeWidget(BaseWidget):

    """
    Widget that renders the datetime as two separate boxes, using jQuery
    to bind the appropriate client side widgets.
    The following attrs are supported:

     * date_format Defaults to dd-mm-YYYY
     * time_format Defaults to hh:mm
    """

    template_name = 'djinn_forms/snippets/datetimewidget.html'
    defaults = {'date_format': '%d-%m-%Y', 'time_format': '%H:%M'}

    def build_attrs(self, extra_attrs=None, **kwargs):

        final_attrs = super(DateTimeWidget, self).build_attrs(
            extra_attrs=extra_attrs, **kwargs)

        if kwargs.get('value'):
            final_attrs['date_value'] = \
                kwargs['value'].strftime(self.attrs['date_format'])
            final_attrs['time_value'] = \
                kwargs['value'].strftime(self.attrs['time_format'])
        else:
            final_attrs['date_value'] = ""
            final_attrs['time_value'] = ""

        return final_attrs

    def value_from_datadict(self, data, files, name):

        value = None

        if data.get("%s_date" % name):

            value_str = data['%s_date' % name]
            format_str = self.attrs['date_format']

            if data.get("%s_time" % name):

                value_str = "%s %s" % (value_str, data['%s_time' % name])
                format_str = "%s %s" % (format_str, self.attrs['time_format'])

                value = datetime.strptime(value_str, format_str)

        return value



from djinn_forms.fields.relate import RelateField


class RelateMixin(object):

    """ When using relate fields, handle these in the save of your
    form. """

    def save_relations(self, commit=True):

        """ Call this method anywhere in your save override """

        for f_name, field in self.fields.items():
            if isinstance(field, RelateField):

                val = self.cleaned_data.get(f_name, {'rm': [], 'add': []})
                field.save_relations(self.instance, val, commit)

    def init_relation_fields(self):

        """ Call this in init, but AFTER calling super init """

        for f_name, field in self.fields.items():
            if isinstance(field, RelateField):
                self.fields[f_name].instance = self.instance



from django.forms.widgets import Widget
from django.utils.safestring import mark_safe
from django.template.loader import render_to_string


class BaseWidget(Widget):

    defaults = {}

    def __init__(self, attrs=None):

        _attrs = self.defaults.copy()

        if attrs:
            _attrs.update(attrs)

        super(BaseWidget, self).__init__(attrs=_attrs)

    @property
    def template_name(self):

        raise NotImplementedError

    def build_attrs(self, extra_attrs=None, **kwargs):

        return super(BaseWidget, self).build_attrs(
            extra_attrs=extra_attrs, **kwargs)

    """ Base widget that renders a template """

    def render(self, name, value, attrs=None):

        final_attrs = self.build_attrs(attrs, name=name, value=value)

        html = render_to_string(self.template_name, final_attrs)

        return mark_safe(u"".join(html))


class InOutWidget(BaseWidget):

    """ Widget that handles data with add and remove lists. The
    incoming data should have <name>_add and <name>_rm values,
    sepratated by 'separator'. """

    separator = ";;"

    def convert_item(self, item):

        """ Convert a single incoming value value to the actual value """

        return item

    def value_from_datadict(self, data, files, name):

        """ The data may contain a list of objects to remove, and
        objects to add. Both are prefixed by the field name. The
        returned value is a dict with 'rm' and 'add' lists, that list
        the """

        result = {'rm': [], 'add': []}

        for item in data.get("%s_rm" % name, "").split(self.separator):

            try:
                obj = self.convert_item(item)
                result['rm'].append(obj)
            except:
                pass

        for item in data.get("%s_add" % name, "").split(self.separator):

            try:
                obj = self.convert_item(item)
                result['add'].append(obj)
            except:
                pass

        return result



from django.forms.fields import Field
from djinn_forms.widgets.relate import RelateWidget, RelateSingleWidget
from djinn_core.utils import object_to_urn
from base import AdditionalHandlingMixin


class UpdateRelations(object):

    def __init__(self, instance, data, relation_type):
        self.instance = instance
        self.rms = data.get('rm', [])
        self.adds = data.get('add', [])
        self.relation_type = relation_type

    def update(self):

        # Unrelate
        for tgt in self.rms:

            #if callable(self.relation_type):
            #    rtype = self.relation_type(tgt)
            #else:
            rtype = self.relation_type

            self.instance.rm_relation(rtype, tgt)

        # Relate
        for tgt in self.adds:

            #if callable(self.relation_type):
            #    rtype = self.relation_type(tgt)
            #else:
            rtype = self.relation_type

            self.instance.add_relation(rtype, tgt)


class UpdateRelation(object):

    """ Updater for single relation. Existing relations are deleted """

    def __init__(self, instance, tgt, relation_type):
        self.instance = instance
        self.tgt = tgt
        self.relation_type = relation_type
        
    def update(self):
        if self.tgt:
            self.instance.get_relations(
                relation_type_list=[self.relation_type]).delete()
            self.instance.add_relation(self.relation_type, self.tgt)


class RelateField(Field, AdditionalHandlingMixin):

    """ Field for relations, based on """

    widget = RelateWidget
    updater = UpdateRelations

    def __init__(self, relation_type, content_types, *args, **kwargs):

        self.relation_type = relation_type
        self.content_types = content_types
        self.instance = None

        super(RelateField, self).__init__(*args, **kwargs)

    def save_relations(self, obj, data, commit):

        """ Save relations given in the data, by keys 'rm' and 'add' """

        relation_updater = self.updater(obj, data, self.relation_type)

        if commit:
            relation_updater.update()
        else:
            # append the updater instance to the object. Note that it's a list
            # since there can be more than one relation field per instance
            if not hasattr(obj, '_relation_updater'):
                obj._relation_updater = []
            obj._relation_updater.append(relation_updater)

    def widget_attrs(self, widget):

        """Renders this field as an HTML widget."""

        attrs = super(RelateField, self).widget_attrs(widget)

        attrs.update({'content_types': self.content_types})

        return attrs

    def prepare_value(self, data):

        """ Return relations for this field. If data is empty,
        we simply get the related objects for the given type.
        If data['rm'] has values, filter those out of the result.
        If data['add'] has values, add those to the result
        """

        relations = self.instance.get_related(self.relation_type)

        try:
            relations = filter(lambda x: x not in data['rm'], relations)
        except:
            pass

        try:
            relations += data['add']
        except:
            pass

        return [{'label': rel.title, 'value': object_to_urn(rel)} for rel in
                relations]


class RelateSingleField(RelateField):

    widget = RelateSingleWidget
    updater = UpdateRelation

    def prepare_value(self, data):

        """ Get existing role, if it's there """

        value = super(RelateSingleField, self).prepare_value(data)

        if value:
            return value[0]
        else:
            return None



from django.forms.fields import Field
from djinn_forms.widgets.share import ShareWidget
from djinn_core.utils import object_to_urn


class ShareField(Field):

    """ Field for shares, inspired by RelateField """

    widget = ShareWidget

    def __init__(self, mode, content_types, *args, **kwargs):

        self.mode = mode
        self.content_types = content_types
        self.instance = None

        super(ShareField, self).__init__(*args, **kwargs)

    def save_share(self, obj, data, commit):

        """ Save shares given in the data, by keys 'rm' and 'add' """

        class UpdateShare(object):

            def __init__(self, instance, rms, adds, mode):
                self.instance = instance
                self.rms = rms
                self.adds = adds
                self.mode = mode

            def update(self):
                # Revoke share
                for tgt in self.rms:
                    self.instance.rm_share(tgt.ct_name, tgt.id, self.mode)
                # share
                for tgt in self.adds:
                    self.instance.add_share(tgt.ct_name, tgt.id, self.mode)

        share_updater = UpdateShare(
            obj, data.get('rm', []), data.get('add', []), self.mode)

        if commit:
            share_updater.update()

        else:

            # append the updater instance to the object. Note that it's a list
            # since there can be more than one relation field per instance
            if not hasattr(obj, '_share_updater'):
                obj._share_updater = []
            obj._share_updater.append(share_updater)

    def widget_attrs(self, widget):

        """Renders this field as an HTML widget."""

        attrs = super(ShareField, self).widget_attrs(widget)

        attrs.update({'content_types': self.content_types})

        return attrs

    def prepare_value(self, data):

        """ Return shares for this field. If data is empty,
        we simply get the related objects for the given type.
        If data['rm'] has values, filter those out of the result.
        If data['add'] has values, add those to the result
        """

        shares = self.instance.shares.all()

        try:
            shares = filter(lambda x: x not in data['rm'], shares)
        except:
            pass

        try:
            shares += data['add']
        except:
            pass

        prepared = []
        for rel in shares:
            if rel.user_id:
                uprofile = rel.user.profile
                prepared.append({'label': str(uprofile), 'value': object_to_urn(uprofile)})
            if rel.usergroup_id:
                gprofile = rel.usergroup.profile
                prepared.append({'label': str(gprofile), 'value': object_to_urn(gprofile)})
        return prepared



class AdditionalHandlingMixin(object):

    """ Marker class to allow for extra handling after save """

    def handle(self, form):

        """ Do extra handling that may or may not be related to an instance """

        pass
